#!/usr/bin/perl

use File::Path;
use File::Copy;
use File::Glob ":glob";
use POSIX ":sys_wait_h";

$timeout = 60;

$ENV{HOME}="/root" if !defined $ENV{HOME};
$ENV{TMP}="/tmp";
$ENV{MOZILLA_FIVE_HOME}="/usr/lib/mozilla-firefox-1.0.6";
$ENV{LD_LIBRARY_PATH}="/usr/lib/mozilla-firefox-1.0.6";
$ENV{MOZ_DISABLE_GNOME}="1";

umask 022;

if ( -f "/usr/lib/mozilla-firefox-1.0.6/mozilla-firefox-bin" )
{
    # remove all of the old files
    rmtree("/usr/lib/mozilla-firefox-1.0.6/chrome/overlayinfo");
#    unlink </usr/lib/mozilla-firefox-1.0.6/chrome/*.rdf>;

    # change cache date to cause it to be dirty 
    system("touch /usr/lib/mozilla-firefox-1.0.6/chrome/chrome.rdf");
    unlink("/usr/lib/mozilla-firefox-1.0.6/components.ini");
    unlink("/usr/lib/mozilla-firefox-1.0.6/defaults.ini");
    unlink </usr/lib/mozilla-firefox-1.0.6/components/*.dat>;

    # create a new clean path
    mkpath("/usr/lib/mozilla-firefox-1.0.6/chrome/overlayinfo");

    # rebuild the installed-chrome.txt file from the installed
    # languages
    if ( -f "/usr/lib/mozilla-firefox-1.0.6/chrome/lang/installed-chrome.txt" ) {
	rebuild_lang_files();
    }

    # Exec the "generate-chome.sh" script so that installed-chrome.txt gets re-generated
    $pid = fork();

    # I am the child.
    if ($pid == 0) {
    	exec("cd ".$ENV{MOZILLA_FIVE_HOME}."/chrome/rc.d && ./generate-chrome.sh");
    }
    # I am the parent.
    else {
	my $timepassed = 0;
	do {
	    $kid = waitpid($pid, &WNOHANG);
	    sleep(1);
	    $timepassed++;
        } until $kid == -1 || $timepassed > $timeout;

	# should we kill?
	if ($timepassed > $timeout) {
	    kill (9, $pid);
	    # kill -9 can leave threads hanging around
	    system("/usr/bin/killall -9 ./generate-chrome.sh");
	}
    }

    # make sure installed-chrome.txt is always more recent than chrome.rdf
    system("touch /usr/lib/mozilla-firefox-1.0.6/chrome/installed-chrome.txt");

    # Register firefox
    $pid = fork();

    # I am the child.
    if ($pid == 0) {
	exec("/usr/lib/mozilla-firefox-1.0.6/mozilla-firefox-bin -register> /dev/null 2> /dev/null");
    }
    # I am the parent.
    else {
	my $timepassed = 0;
	do {
	    $kid = waitpid($pid, &WNOHANG);
	    sleep(1);
	    $timepassed++;
        } until $kid == -1 || $timepassed > $timeout;

	# should we kill?
	if ($timepassed > $timeout) {
	    kill (9, $pid);
	    # kill -9 can leave threads hanging around
            system("/usr/bin/killall -9 /usr/lib/mozilla-firefox-1.0.6/mozilla-firefox-bin");
	}
    }


}



sub rebuild_lang_files {
    unlink("/usr/lib/mozilla-firefox-1.0.6/chrome/installed-chrome.txt");

    open (OUTPUT, "+>", "/usr/lib/mozilla-firefox-1.0.6/chrome/installed-chrome.txt")||
	die("Failed to open installed-chrome.txt: $!\n");

    copy("/usr/lib/mozilla-firefox-1.0.6/chrome/lang/installed-chrome.txt",
	 \*OUTPUT);

    foreach (bsd_glob("/usr/lib/mozilla-firefox-1.0.6/chrome/lang/lang-*.txt")) {
	copy($_, \*OUTPUT);
    }

    copy("/usr/lib/mozilla-firefox-1.0.6/chrome/lang/default.txt",
	 \*OUTPUT);
}
